#include "types.h"
#include "bioscall.h"
#include "config.h"


int bios_call(unsigned long id, unsigned long arg)
{
	BIOSCALL *bios_call_ptr = (BIOSCALL *)BIOS_API_ADDR;
	return bios_call_ptr(id, arg);
}

int bios_reboot()
{
	return bios_call(BIOSCALL_REBOOT, 0);
}

unsigned short bios_vendor_id(void)
{
	unsigned short vendor_id;

	if(bios_call(BIOSCALL_GET_VENDORID, (unsigned long)&vendor_id) == -1)
		return -1;
	return vendor_id;
}

unsigned short bios_device_id(void)
{
	unsigned short device_id;

	if(bios_call(BIOSCALL_GET_DEVICEID, (unsigned long)&device_id) == -1) 
		return -1;
	return device_id;
}

unsigned long bios_rev(void)
{
	unsigned long rev;

	if(bios_call(BIOSCALL_GET_REV, (unsigned long)&rev) == -1)
		return -1;
	return rev;
}

unsigned long bios_bios_table_offset(void)
{
	unsigned long bios_table_offset;

	if(bios_call(BIOSCALL_GET_BIOSTABLEOFFSET, (unsigned long)&bios_table_offset) == -1)
		return -1;	 
	return bios_table_offset;
}

unsigned long bios_safe_table_offset(void)
{
	unsigned long safe_table_offset;

	if(bios_call(BIOSCALL_GET_SAFETABLEOFFSET, (unsigned long)&safe_table_offset) == -1)
		return -1;
	return safe_table_offset;
}

unsigned long bios_bios_offset(void)
{
	unsigned long bios_offset;

	if(bios_call(BIOSCALL_GET_BIOSOFFSET, (unsigned long)&bios_offset) == -1)
		return -1;
	return bios_offset;
}

unsigned long bios_bios_size(void)
{
	unsigned long bios_size;

	if(bios_call(BIOSCALL_GET_BIOSSIZE, (unsigned long)&bios_size) == -1)
		return -1;
	return bios_size;
}

unsigned long bios_rom_base(void)
{
	unsigned long rom_base;

	if(bios_call(BIOSCALL_GET_ROMBASE, (unsigned long)&rom_base) == -1)
		return -1; 
	return rom_base;
}

unsigned long bios_rom_size(void)
{
	unsigned long rom_size;

	if(bios_call(BIOSCALL_GET_ROMSIZE, (unsigned long)&rom_size) == -1)
		return -1;
	return rom_size;
}

unsigned long bios_dram_base(void)
{
	unsigned long dram_base;

	if(bios_call(BIOSCALL_GET_DRAMBASE, (unsigned long)&dram_base) == -1)
		return -1;
	return dram_base;
}

unsigned long bios_dram_size(void)
{
	unsigned long dram_size;

	if(bios_call(BIOSCALL_GET_DRAMSIZE, (unsigned long)&dram_size) == -1)
		return -1;
	return dram_size;
}

unsigned long bios_startup_mode(void)
{
	unsigned long startup_mode;

	if(bios_call(BIOSCALL_GET_STARTUPMODE, (unsigned long)&startup_mode) == -1)
		return -1;
	return startup_mode;
}

unsigned long bios_get_userflag(void)
{
	unsigned long userflag;

	if(bios_call(BIOSCALL_GET_USERFLAG, (unsigned long)&userflag) == -1)
		return -1;
	return userflag;
}

unsigned long bios_set_userflag(unsigned long userflag)
{
	return bios_call(BIOSCALL_SET_USERFLAG, userflag);
}

int bios_flash_erase(unsigned long adr, unsigned long size)
{
	struct flash_erase_struct flash_erase;

	flash_erase.addr = adr;
	flash_erase.size = size;
	return bios_call(BIOSCALL_FLASH_ERASE, (unsigned long)&flash_erase);
}

int bios_flash_read(unsigned long from,
	unsigned long len, unsigned long *retlen, unsigned char *buf)
{
	struct flash_read_struct flash_read;
	int ret;

	flash_read.from = from;
	flash_read.len = len;
	flash_read.buf = buf;
	ret = bios_call(BIOSCALL_FLASH_READ, (unsigned long)&flash_read);
	*retlen = flash_read.retlen;

	return ret;
}

int bios_flash_write(unsigned long to,
	unsigned long len, unsigned long *retlen, unsigned char *buf)
{
	struct flash_write_struct flash_write;
	int ret;

	flash_write.to = to;
	flash_write.len = len;
	flash_write.buf = buf;
	ret = bios_call(BIOSCALL_FLASH_WRITE, (unsigned long)&flash_write);
	*retlen = flash_write.retlen;

	return ret;
}

int bios_bios_table(struct bios_table_struct *bios_table)
{
	unsigned long bios_table_offset, l;

	bios_table_offset = bios_bios_table_offset();
	if(bios_table_offset == (unsigned long) -1)
		return -1;

	return bios_flash_read(bios_table_offset, sizeof(struct bios_table_struct),
			&l, (unsigned char *)bios_table);
}

int bios_safe_table(struct bios_table_struct *safe_table)
{
	unsigned long safe_table_offset, l;

	safe_table_offset = bios_safe_table_offset();
	if(safe_table_offset == (unsigned long) -1) 
		return -1;

	return bios_flash_read(safe_table_offset, sizeof(struct bios_table_struct),
			&l, (unsigned char *)safe_table);
}
	
int bios_eth_cfg(struct sys_eth_cfg *eth)
{
	struct bios_table_struct *bios_table;
	unsigned long from, l;

	bios_table = (struct bios_table_struct *)bios_bios_table_offset();
	if(bios_table == (struct bios_table_struct *) -1) 
		return -1;
	from = (unsigned long)&(bios_table->system_table.eth);
	
	return bios_flash_read(from, sizeof(struct sys_eth_cfg), &l, (unsigned char *)eth);
}
