#include "types.h"
#include "console.h"
#include "utils.h"
#include "board.h"

#ifndef __STDARG_H
#define __STDARG_H

typedef char *va_list;
#define va_start(ap, p)		(ap = (char *) (&(p)+1))
#define va_arg(ap, type)	((type *) (ap += sizeof(type)))[-1]
#define va_end(ap)
#endif /* __STDARG_H */

#define _S3C4530_

#ifdef _S3C4530_ 
#define UCON0_4530	(BASE_ADDR+0xd000) /* UART0 control regisger */
#define UCON1_4530	(BASE_ADDR+0xe000) /* UART1 contgrol register */
#define USTAT0_4530	(BASE_ADDR+0xd004) /* UART0 status register */
#define USTAT1_4530	(BASE_ADDR+0xe004) /* UART1 status register */
#define UINTEN0_4530    (BASE_ADDR+0xd008) /* UART0 interrupt enable register */
#define UINTEN1_4530    (BASE_ADDR+0xe008) /* UART1 interrupt enable register */

#define USTAT_4530_RDV 		0x00000001  /* rx data */
#define USTAT_4530_THE		0x00040000  /* transmit holding register empty */

#define	UCON_4530_TMODE_INTREQ  0x00000001
#define	UCON_4530_RMODE_INTREQ  0x00000004
#define	UCON_WL8                0x00003000
#endif 							/* _S3C4530_ */

#define CONSOLE_OUTPUT            1

int console_init(void)
{
#ifndef _S3C4530_ 
	outl(0x03, ULCON0);
	outl(0x09, UCON0);
	outl(0x500, UBRDIV0);	/* 19200bps */
//	outl(0x1a0, UBRDIV0);	/* 57600bps */

	outl(0, ULCON1);
	outl(0, UCON1);
	outl(0, UBRDIV1);
#else 
	if( (inl(SYSCFG) & 0x0C000000) == 0x0C000000 ) {
	    outl((UCON_4530_TMODE_INTREQ | UCON_4530_RMODE_INTREQ | UCON_WL8), UCON0_4530);
	    outl(0, UCON1_4530);
	    outl(0, UINTEN0_4530);
	    outl(0, UINTEN1_4530);
	} else {
	    outl(0x03, ULCON0);
	    outl(0x09, UCON0);

	    outl(0, ULCON1);
	    outl(0, UCON1);
	 } 
	outl(0x500, UBRDIV0);	/* 19200bps */
//	outl(0x1a0, UBRDIV0);	/* 57600bps */
	outl(0, UBRDIV1);
#endif 							/* _S3C4530_ */
	return 0;
}

int console_write(int ch)
{
#ifndef _S3C4530_ 
#ifdef CONSOLE_OUTPUT
	while (!(inl(USTAT0) & 0x40));
	outl(ch, UTXBUF0);
#endif
#else 
#ifdef CONSOLE_OUTPUT
	unsigned long reg, msk;
	if( (inl(SYSCFG) & 0x0C000000) == 0x0C000000 ) {
	    reg = USTAT0_4530;
	    msk = USTAT_4530_THE;
	} else {
	    reg = USTAT0;
	    msk = 0x40;
	}

	while (!(inl(reg) & msk));
	outl(ch, UTXBUF0);
#endif
#endif 							/* _S3C4530_ */
	return 0;
}

int console_read(void)
{
#ifndef _S3C4530_ 
	while (!(inl(USTAT0) & 0x20));
#else 
	unsigned long reg, msk;
	if( (inl(SYSCFG) & 0x0C000000) == 0x0C000000 ) {
	    reg = USTAT0_4530;
	    msk = USTAT_4530_RDV;
	} else {
	    reg = USTAT0;
	    msk = 0x20;
	}
	while (!(inl(reg) & msk));
#endif 							/* _S3C4530_ */
	return (inl(URXBUF0) & 0xFF);
}

int kbhit(void)
{
#ifndef _S3C4530_ 
	return (inl(USTAT0) & 0x20);
#else 
	if( (inl(SYSCFG) & 0x0C000000) == 0x0C000000 ) {
	    return ( inl(USTAT0_4530) & USTAT_4530_RDV);
	} else {
	    return ( inl(USTAT0) & 0x20 );
	}
#endif 							/* _S3C4530_ */
}


int getch(void)
{
	return console_read();
}

int putch(int ch)
{
	return console_write(ch);
}

int puts(const char *s)
{
	while (*s != 0) {
		putch(*s++);
	}
	return 0;
}

char *getpass(char *s, int n, char prompt)
{
	char *p;
	int ch;

	p = s;
	while (1) {
		ch = getch();
		if (ch == KEY_ENTER)
			break;
		if (ch == KEY_BACKSPACE) {
			if (p > s) {
				p--;
				putch(ch);
				putch(' ');
				putch(ch);
			}
		} else if ((p - s) < (n - 1)) {
			/* visiable ascii chars */
			if (ch >= 32 && ch <= 126) {
				*p++ = (char)ch;
				if (prompt)
					putch(prompt);
			}
		}
	}
	*p = '\0';

	return s;
}

char *gets(char *s, int n)
{
	char *p;
	int ch;

	p = s;
	while (1) {
		ch = getch();
		if (ch == KEY_ENTER)
			break;
		if (ch == KEY_BACKSPACE) {
			if (p > s) {
				p--;
				putch(ch);
				putch(' ');
				putch(ch);
			}
		} else if ((p - s) < (n - 1)) {
			/* visiable ascii chars */
			if (ch >= 32 && ch <= 126) {
				*p++ = (char)ch;
				putch(ch);
			}
		}
	}
	*p = '\0';

	return s;
}

unsigned long getul(unsigned long def, int radix)
{
	char buf[9];

	gets(buf, 9);
	if (buf[0] == '\0')
		return def;
	return strtoul(buf, NULL, radix);
}

unsigned char getuc(unsigned char def, int radix)
{
	char buf[4];
	unsigned long l;
	
	if (radix == 16)
		gets(buf, 3);
	else
		gets(buf, 4);

	if (buf[0] == '\0')
		return def;

	l = strtoul(buf, NULL, radix);
	if (l > 255)
		l = 255;

	return l;
}	

void print_0nx(char ch2, unsigned long l)
{
	unsigned char ch;
	int i;

	ch2 = ch2 - '0';
	for (i = ch2 - 1; i >= 0; i--) {
		ch = (l >> (i * 4)) & 0x0f;
		if (ch < 10)
			putch(ch + '0');
		else
			putch(ch - 10 + 'a');
	}
}

void print_0nd(char ch2, unsigned long l)
{
	unsigned long prod, t;
	int flag = 0;
	unsigned char ch;
	int i, j;

	ch2 = ch2 - '0';
	for (i = 15; i >= 0; i--) {
		prod = 1;
		t = l;
		for (j = 0; j < i; j++) {
			prod = prod * 10;
			t = t / 10;
		}
		ch = t;
		l -= prod * t;

		if (!flag && !ch && i >= ch2)
			continue;
		
		if (ch < 10)
			putch(ch + '0');
		else
			putch('?');

		flag = 1;
	}
}


int printf(const char *s, ...)
{
	va_list ap;
	unsigned long arg;
	const char *str;
	char ch1, ch2, ch3;

	va_start(ap, s);
	while (*s != 0) {
		if (*s == '%') {
			s++;
			ch1 = *s++;
			if (ch1 == 'l')
				ch1 = *s++;
			if (ch1 == 's') {
				str = va_arg(ap, unsigned char *);
				puts(str);
			} else if (ch1 == 'd') {
				arg = va_arg(ap, unsigned long);
				print_0nd('1', arg);
			} else {
				ch2 = *s++;
				ch3 = *s++;
				if (ch3 == 'l')
					ch3 = *s++;
				arg = va_arg(ap, unsigned long);
				if (ch3 == 'd')
					print_0nd(ch2, arg);
				else
					print_0nx(ch2, arg);
			}
		} else
			putch(*s++);
	}
	va_end(ap);
	return 0;
}

int get_select(int min, int max)
{
	int select;
	int ch;

	while (1) {
		ch = getch();
		if (ch >= min && ch <= max) {
			putch(ch);
			select = ch - min;
			break;
		}
		if (ch == KEY_ESC) {
			select = -1;
			break;
		}
	}

	return select;
}

int get_select_enter(int min, int max)
{
	int select;
	int ch;

	while (1) {
		ch = getch();
		if (ch >= min && ch <= max) {
			putch(ch);
			select = ch - min;
			break;
		}
		if (ch == KEY_ENTER) {
			select = -1;
			break;
		}
	}

	return select;
}

int get_yes_no(int def)
{
	int ch;
	
	while (1) {
		ch = getch();
		if (ch == KEY_ENTER)
			return def;
		if (ch == 'y' || ch == 'Y') {
			putch(ch);
			return 1;
		}
		if (ch == 'n' || ch == 'N') {
			putch(ch);
			return 0;
		}
	}

	return def;
}

int get_enter_esc(void)
{
	int ch;

	while (1) {
		ch = getch();
		if (ch == KEY_ENTER)
			return 1;
		if (ch == KEY_ESC)
			return 0;
	}

	return 0;
}
