/* System table settings */
#define tSYSCFG ( \
	SYS_ALWAYS_SET | \
	SYS_CACHE_ON | \
	SYS_WBUF_ON | \
	SYS_CACHE_8K \
)

#define tCLKCON ( \
	CLK_ALWAYS_SET | \
	CLK_DIV_1 \
)

#define tEXTACON ( \
	EXT_ALWAYS_SET | \
	EXT_TCOS_7CYCLE | \
	EXT_TACS_7CYCLE | \
	EXT_TCOH_7CYCLE | \
	EXT_TACC_7CYCLE \
)

#define tROMCON ( \
	ROM_ALWAYS_SET | \
	ROM_PMC_NORMAL | \
	ROM_TPA_2CYCLE | \
	ROM_TACC_3CYCLE \
)

#define tDRAMCON ( \
	DRAM_ALWAYS_SET | \
	DRAM_TRCD_1CYCLE | \
	DRAM_TRP_2CYCLE | \
	DRAM_CAN_8BIT \
)
		
#define tREFEXTCON ( \
	REF_ALWAYS_SET | \
	REF_TRC_1CYCLE | \
	REF_PERIOD_16US \
)

#define tIOPMOD	(0x00000080)
#define tIOPCON (0x00000000)
#define tIOPDATA (~0x00000080)

#define ETH_MAC_ADDR	{0x00, 0x20, 0xed, 0x70, 0xe0, 0x90}
#define ETH_IP_ADDR	0xc0a80064	/* 192.168.0.100 */


struct bios_table_struct bios_table = {
system_table:
{
	vendor_id:	GENERAL_VENDOR_ID,
	device_id:	GENERAL_DEVICE_ID,
	rev:		0,

	bios_table_offset:	BIOS_TABLE_OFFSET,
	safe_table_offset:	SAFE_TABLE_OFFSET,

	/*
	 * pmc index: 0=normal rom, 1=4word, 2=8word, 3=16word
	 * tpa index: 0=2cycle, 1=3cycle, 2=4cycle, 3=5cycle
	 * tacc index: 0=0cycle, 1=2cycle, 2=3cycle, 3=4cycle, 4=7cycle
	 */
	rom_table:      
	/* size       width        flag     pmc  tpa  tacc*/
	{{0x00080000, DBWTH_8BIT,  tROMCON, 0,   0,   2},
	 {0x00200000, DBWTH_16BIT, tROMCON, 0,   0,   2},
	 {0x00000000, DBWTH_8BIT,  tROMCON, 0,   3,   4},
	 {0x00000000, DBWTH_8BIT,  tROMCON, 0,   3,   4},
	 {0x00000000, DBWTH_8BIT,  tROMCON, 0,   3,   4},
	 {0x00000000, DBWTH_8BIT,  tROMCON, 0,   3,   4}},

	 
	/*
	 * trcd index: 0=1cycle, 1=2cycle
	 * trp index: 0=1cycle, 1=2cycle, 2=3cycle, 3=4cycle
	 * can index: 0=8bit, 1=9bit, 2=10bit, 3=11bit
	 * trc index: 0=1cycle, 1=2cycle, 2=3cycle, 3=4, 4=5, 5=6
	 * period index: 0=8us, 1=16us, 2=24us, 3=32us
	 */
	dram_table:
	/* size       width        flag      refresh     trcd  trp  can  trc  period */
	{{0x01000000, DBWTH_32BIT, tDRAMCON, tREFEXTCON, 0,    1,   0,   0,   1},
	 {0x00000000, DBWTH_32BIT, tDRAMCON, tREFEXTCON, 1,    3,   0,   5,   1},
	 {0x00000000, DBWTH_32BIT, tDRAMCON, tREFEXTCON, 1,    3,   0,   5,   1},
	 {0x00000000, DBWTH_32BIT, tDRAMCON, tREFEXTCON, 1,    3,   0,   5,   1}},


	/*
	 * tcos index: 0=0cycle, 1=1, 2=2, 3=3, 4=4, 5=5, 6=6, 7=7
	 * tacs index: 0=0cycle, 1=1, 2=2, 3=3, 4=4, 5=5, 6=6, 7=7
	 * tcoh index: 0=0cycle, 1=1, 2=2, 3=3, 4=4, 5=5, 6=6, 7=7
	 * tacc index: 0=0cycle, 1=1, 2=2, 3=3, 4=4, 5=5, 6=6, 7=7
	 */
	ext_table:
	/* width       flag      tcos  tacs  tcoh  tacc */
	{{DBWTH_32BIT, tEXTACON, 7,    7,    7,    7},
	 {DBWTH_32BIT, tEXTACON, 7,    7,    7,    7},
	 {DBWTH_32BIT, tEXTACON, 7,    7,    7,    7},
	 {DBWTH_32BIT, tEXTACON, 7,    7,    7,    7}},

	 
	/*
	 * cache index: 0=0k(off), 1=4k(on), 2=8k(on)
	 * wbuf index: 0=off, 1=on
	 * clk_div index: 0=div1, 1=div2, 2=div3, 3=div5
	 */
	 		/* SYSCFG  CLKCON   cache  wbuf  clk_div */
	cpu:		{tSYSCFG,  tCLKCON, 2,     1,    0},

	iop:            {tIOPMOD, tIOPCON, tIOPDATA},
	eth:            {ETH_MAC_ADDR, ETH_IP_ADDR},
			
	bios_size:	BIOS_SIZE,
	startup_mode:   BOOT_MENU,

	password:	{'a', 'r', 'm', '\0'},
},

#define PF_NONE		PART_FLAG_NONE
#define PF_DISK		PART_FLAG_DISK
#define PF_NET		PART_FLAG_NET
#define PT_RO		PART_TYPE_RO
#define PT_RW		PART_TYPE_RW
#define IMG_OTH		IMAGE_OTHER
#define IMG_LIN		IMAGE_UCLINUX
#define LIN_EADDR	UCLINUX_EXEC_ADDR
#define LIN_PADDR	UCLINUX_PARAM_ADDR

partition_table:
{
boot_partition:		1,
partition:
/* flag,   type,    offset,     size,   image, gzipped, exec_addr, param_addr, cmdline */
{{PF_DISK, PT_RO,   0x00000000, 0x00020000, IMG_OTH, 0, 0,         0,         '\0'},
 {PF_DISK, PT_RW,   0x00020000, 0x00080000, IMG_LIN, 1, LIN_EADDR, LIN_PADDR, '\0'},
 {PF_DISK, PT_RW,   0x000a0000, 0x001e0000, IMG_LIN, 0, LIN_EADDR, LIN_PADDR, '\0'},
 {PF_NONE, PT_RW,   0x00000000, 0x00000000, IMG_LIN, 0, LIN_EADDR, LIN_PADDR, '\0'},
 {PF_NONE, PT_RW,   0x00000000, 0x00000000, IMG_LIN, 0, LIN_EADDR, LIN_PADDR, '\0'},
 {PF_NONE, PT_RW,   0x00000000, 0x00000000, IMG_LIN, 0, LIN_EADDR, LIN_PADDR, '\0'},
 {PF_NONE, PT_RW,   0x00000000, 0x00000000, IMG_LIN, 0, LIN_EADDR, LIN_PADDR, '\0'},
 {PF_NET,  PT_RO,   0x00000000, 0x00000004, IMG_LIN, 0, LIN_EADDR, LIN_PADDR, '\0'}
},
},

};	/* bios_table */

