#include <stdio.h>
#include <stdlib.h>
#include "bios.h"
#include "board.h"
#include "config.h"

/* include your board parameters */
//#include "./boards/common.h"
#include "./boards/evm50100.h"
//#include "./boards/bluebird.h"

/* comment to disable checking */
#define CHECK_BIOS_TABLE
/* uncomment to ignore errors */
//#define IGNORE_ERROR

#ifdef CHECK_BIOS_TABLE
/* bios table ckecker */ 
static int check_bios_table(struct bios_table_struct * bios_table);
#endif 							/* CHECK_BIOS_TABLE */

int main()
{
	FILE *f;

	printf("size = %d\n", sizeof(struct bios_table_struct));

	f = fopen("bios.tbl", "wb");
	fwrite((char *)(&bios_table), sizeof(struct bios_table_struct), 1, f);

	fclose(f);
#ifdef CHECK_BIOS_TABLE
	if(check_bios_table(&bios_table) == 0) {
	    printf("Bios table is writed to file 'bios.tbl', but contain errors\n");
#ifdef IGNORE_ERROR
	    printf("Ignored\n");
	    return(0);
#else
	    return(1);
#endif
	}
#endif 							/* CHECK_BIOS_TABLE */
	return 0;
}

#ifdef CHECK_BIOS_TABLE

#define  ARRAY_ONLY			
#include "../../setup/setup.c"

// return size of array value 

#define setup_size(value) (sizeof(value) / sizeof(value[0])) 

// reg  = register name 
// fld  = field name 

static int setup_must_be(char * reg, char * fld, const unsigned long * value,  const char ** text, int size)
{
    int in;
    if(reg == 0 || fld == 0 || value == 0 || text == 0) {
// null register name 'reg' or field name 'fld' or text array
	return(0);
    }

    fprintf(stderr, "[%s] of %s must be:\n%-8s %-8s string\n", fld, reg, "index","value");
    for(in = 0; in < size; ++in) {
	if(*(text + in) != 0) {
	    fprintf(stderr, "  %-4d %08x %s\n", in, *(value + in), *(text + in));
	}
    }
// 
    return(1);
}

// check width field 
// reg = register name 
// width = bus width 

static int setup_width(char * reg, int width)
{
 if(reg == 0) {				// null pointer to register name
    return(0);		
 }

 if( 
    ((width == DBWTH_8BIT ) || 
     (width == DBWTH_16BIT) ||
     (width == DBWTH_32BIT)) == 0
 ) {
    fprintf(stderr,
        "%s bus width error [%d],\n"
        "must be:\n"
        " 1 -  8 Bit\n"
        " 2 - 16 Bit\n"
        " 3 - 32 Bit\n",
        reg, width
    );
    return(0);
 }
//
 return(1);
}

/*
    Check register field value  
    flag  = register value
    mask  = field mask
    fld   = field index in array 'value' 
    value = array of field valid values 
    text  = text description 
    size  = array size 
    reg   = register name
    nam   = field name
*/

static int setup_index(unsigned long flag, unsigned long mask, unsigned long fld,
const unsigned long * value, const char ** text, int size, char * reg, char * nam)
{    
    if(value == 0 || text == 0 || reg == 0 || nam == 0) {
	return(0);
    }

    if(fld >= size) {
	fprintf(stderr, "%s [%s] index error [%d], max [%d]\n",
	    reg, nam, fld, size - 1
	);

	setup_must_be(reg, nam, value, text, size);
	return(0);
    }
    flag &= mask;

    if( flag != *(value + fld) ) {
	const char * str = "";
	int index;	
	for(index = 0; index < size; ++index) {
	    if( flag == *(value + index) ) {
		str = *(text + index);
		break;
	    }
	}								

	fprintf(stderr, "%s [%s] field not correspond flag field\n", reg, nam);
	fprintf(stderr, "  Value in [%s] field [%d] string [%s]\n", nam, fld, *(text + fld));
	fprintf(stderr, "  Value in flag [%08x] index [%d] string [%s]\n", flag, index, str);
	setup_must_be(reg, nam, value, text, size);
	return(0);
    }
//
  return(1);
}

// 

static int check_bios_table(struct bios_table_struct * bios_table)
{
 int rvl = 1;
 int rn;

 if(bios_table == 0) {
    return(0);
 }

// check ROM banks configuration  
 for(rn = 0; rn < 6; ++rn) {
    char reg[32];
    struct sys_rom_cfg * rc = & (bios_table->system_table.rom_table[rn]);
    if(rc->size <= 0) {
	continue;			// nothing check
    }

    snprintf(reg, sizeof(reg), "ROM%d", rn);

    // chech bus width field
    if(setup_width(reg, rc->width) == 0) {
        rvl = 0;
    }

    // sram/rom options
    if(setup_index(rc->flag, ROM_PMC_16WORD, rc->pmc, pmc_value	, 
	pmc_text, setup_size(pmc_value), reg, "pmc" ) == 0) {
	rvl = 0;
    }

#define ROM_TPA_MASK		(0x00000003 << 2)
    if(setup_index(rc->flag, ROM_TPA_MASK, rc->tpa, tpa_value, 
	tpa_text, setup_size(tpa_value), reg, "tpa") == 0) {
	rvl = 0;
    }

#define ROM_TACC_MASK		(0x00000007 << 4)
    if(setup_index(rc->flag, ROM_TACC_MASK, rc->tacc, tacc_value, 
	tacc_text, setup_size(tacc_value), reg, "tacc") == 0) {
	rvl = 0;
    }
 }    

// check DRAM configuration 
 for(rn = 0; rn < 4; ++rn) {
    struct sys_dram_cfg * dc = & (bios_table->system_table.dram_table[rn]);
    char reg[32];
    if(dc->size <= 0) {
	continue;			// nothing to check
    }

    snprintf(reg, sizeof(reg), "DRAM%d", rn);

    // chech bus width filed
    if(setup_width(reg, dc->width) == 0) {
        rvl = 0;
    }

#define DRAM_TRCD_MASK 	DRAM_TRCD_2CYCLE
    if(setup_index(dc->flag, DRAM_TRCD_MASK, dc->trcd, trcd_value, 
	trcd_text, setup_size(trcd_value), reg, "trcd") == 0) {
	rvl = 0;
    }

#define DRAM_TRP_MASK 	DRAM_TRP_4CYCLE
    if(setup_index(dc->flag, DRAM_TRP_MASK, dc->trp, trp_value, 
	trp_text, setup_size(trp_value), reg, "trp") == 0) {
	rvl = 0;
    }

#define DRAM_CAN_MASK	DRAM_CAN_11BIT
    if(setup_index(dc->flag, DRAM_CAN_MASK, dc->can, can_value, 
	can_text, setup_size(can_value), reg, "can") == 0) {
	rvl = 0;
    }
 }

 {
// check REFEXTCON
    char reg[32];
    struct sys_dram_cfg * dc = & (bios_table->system_table.dram_table[0]);
    snprintf(reg, sizeof(reg), "REFEXTCON");

#define REF_TRC_MASK		(0x00000007 << 17)
    if(setup_index(dc->refresh, REF_TRC_MASK, dc->ref_trc, trc_value, 
	trc_text, setup_size(trc_value), reg, "trc") == 0) {
	rvl = 0;
    }

#define REF_PERIOD_MASK         (0x000007FF << 21)
    if(setup_index(dc->refresh, REF_PERIOD_MASK, dc->ref_period, period_value, 
	period_text, setup_size(period_value), reg, "period") == 0) {
	rvl = 0;
    }
 }

//
 for(rn = 0; rn < 4; ++rn) {
    char reg[32];
    struct sys_ext_cfg * ec = & (bios_table->system_table.ext_table[rn]);
    snprintf(reg, sizeof(reg), "EXTBank%d", rn);

/* chech bus width filed */
    if(setup_width(reg, ec->width) == 0) {
    	    rvl = 0;
    }

/* tcos */
#define EXT_TCOS_MASK EXT_TCOS_7CYCLE
    if(setup_index(ec->flag, EXT_TCOS_MASK, ec->tcos, etcos_value, 
        etcos_text, setup_size(etcos_value), reg, "tcos") == 0) {
        rvl = 0;
    }

/* tacs */
#define EXT_TACS_MASK EXT_TACS_7CYCLE
    if(setup_index(ec->flag, EXT_TACS_MASK, ec->tacs, etacs_value, 
        etacs_text, setup_size(etacs_value), reg, "tacs") == 0) {
        rvl = 0;
    }

/* tcoh */
#define EXT_TCOH_MASK EXT_TCOH_7CYCLE
    if(setup_index(ec->flag, EXT_TCOH_MASK, ec->tcoh, etcoh_value, 
        etcoh_text, setup_size(etcoh_value), reg, "tcoh") == 0) {
        rvl = 0;
    }

/* tacc */
#define EXT_TACC_MASK EXT_TACC_7CYCLE
    if(setup_index(ec->flag, EXT_TACC_MASK, ec->tacc, etacc_value, 
        etacc_text, setup_size(etacc_value), reg, "tacc") == 0) {
        rvl = 0;
    }
 } 

 {
     struct sys_cpu_cfg * cc = & (bios_table->system_table.cpu);

#define SYS_CACHE_MASK ( SYS_CACHE_ON | (0x00000003 << 4) )
    if(setup_index(cc->reg_syscfg, SYS_CACHE_MASK, cc->cache_size, cache_value, 
	cache_text, setup_size(cache_value), "SYSCFG", "Cache size") == 0) {
	rvl = 0;
    }

    if(setup_index(cc->reg_syscfg, SYS_WBUF_ON, cc->wbuf_on, wbuf_value, 
	wbuf_text, setup_size(wbuf_value), "SYSCFG", "Write Buf") == 0) {
	rvl = 0;
    }

    if(setup_index(cc->reg_clkcon, 0x07, cc->clk_div, clk_value, 
	clk_text, setup_size(clk_value), "CLKCON", "ClkDiv") == 0) {
	rvl = 0;
    }
 } 

// check partition table
 {
    struct partition_table_struct * pt = &(bios_table->partition_table); 
    char part_err[8] = {0, 0, 0, 0, 0, 0, 0, 0};
    int romsize = 0, pn;
    if(pt->boot_partition < 0 || pt->boot_partition >= 8) {
	rvl = 0;
	fprintf(stderr, "Incorrect boot partition number [%d]\n",
	    pt->boot_partition);
    }

// calc romsize 
    for(rn = 0; rn < 6; ++rn) {
	struct sys_rom_cfg * rc = & (bios_table->system_table.rom_table[rn]);
	romsize += rc->size;
    }

    for(pn = 0; pn < 8; ++pn) {
	struct partition_struct * pi = pt->partition + pn;
	int pn1;
	if(pi->size == 0) {
	    if(pn == pt->boot_partition) {
		rvl = 0;
		fprintf(stderr, "Size of boot partition [%d] is zero\n", pn);
	    }
	    continue;
	}    

// check partition flag 	
	if(
	   (pi->flag != PART_FLAG_NONE) &&
	   (pi->flag != PART_FLAG_DISK) &&
	   (pi->flag != PART_FLAG_NET)
	) {
	    part_err[pn] = 1;		// mark as 'contain error'
	    rvl = 0; 
//
	    fprintf(stderr, 
	        "Incorrect partition [%d] flag [%d],\n"
	        "must be:\n"
	        " %d - PART_FLAG_NONE\n"
	        " %d - PART_FLAG_DISK\n"
	        " %d - PART_FLAG_NET\n",
		pn, pi->flag	, 
		PART_FLAG_NONE	,
		PART_FLAG_DISK	,
		PART_FLAG_NET
	    );
	}

// check partition type 
	if(
	    (pi->type != PART_TYPE_RO) &&
	    (pi->type != PART_TYPE_RW)
	) {
	    rvl = 0; 
	    part_err[pn] = 1;		// mark as 'contain error'

	    fprintf(stderr, 
	        "Incorrect partition [%d] type [%d],\n"
	        "must be:\n"
	        " %d - PART_TYPE_RO\n"
	        " %d - PART_TYPE_RW\n",
		pn, pi->type	, 
		PART_TYPE_RO	,
		PART_TYPE_RW
	    );
	}

	if(
	    (pi->image_type != IMAGE_OTHER) &&
	    (pi->image_type != IMAGE_UCLINUX)
	) {
	    rvl = 0; 
	    part_err[pn] = 1;
	    fprintf(stderr, 
	        "Incorrect partition [%d] image type [%d],\n"
	        "must be:\n"
	        " %d - IMAGE_OTHER\n"
	        " %d - IMAGE_UCLINUX\n",
		pn, pi->image_type, 
		IMAGE_OTHER	,
		IMAGE_UCLINUX
	    );
	}

	if(pi->offset < 0 || pi->offset >= romsize) {
	    rvl = 0; 
	    part_err[pn] = 1;
	    fprintf(stderr, 
	        "Incorrect partition [%d] offset [0x%08x] must be from 0 to [0x%08x]\n",
		pn, pi->offset, (romsize - 1));
	} else
	if(pi->offset + pi->size > romsize) {
	    rvl = 0;
	    part_err[pn] = 1;
	    fprintf(stderr, 
	        "Incorrect offset [0x%08x] or partition [%d] size [0x%08x],\n"
	        "out of ROM space (0 - 0x%08x)\n",
		pi->offset, pn, pi->size, (romsize-1) );
	}

	if(pi->flag == PART_FLAG_NET) {
	    continue;
	}

//
	for(pn1 = 0; pn1 < pn; ++pn1)  {
	    struct partition_struct * pi1 = pt->partition + pn1;
	    if(part_err[pn1] == 1) {
		continue;
	    }
	    if(pi1->size == 0 || pi1->flag == PART_FLAG_NET) {
		continue;
	    }
//
	    if(
	       (pi->offset >= (pi1->offset + pi1->size) ||
	       (pi->offset + pi->size) <=  pi1->offset) == 0
	    ) {
		rvl = 0;
		part_err[pn] = 1;
		fprintf(stderr, 
		    "Partition [%d] cross partition [%d]\n"
		    " %d 0x%08x - 0x%08x\n"
		    " %d 0x%08x - 0x%08x\n",
		    pn , pn1,
		    pn , pi->offset , pi->offset  + (pi->size - 1), 
		    pn1, pi1->offset, pi1->offset + (pi1->size -1)
		);
	    }	
	}
    }
 }
//
 return(rvl);
}
#endif 							/* CHECK_BIOS_TABLE */

