#include "types.h"
#include "skbuff.h"
#include "eth.h"
#include "arp.h"
#include "ip.h"
#include "udp.h"
#include "tftp.h"
#include "tftpput.h"
#include "utils.h"

static unsigned long  client_ip    = -1;
static unsigned short client_port  = -1;
static unsigned short client_block = -1;

#define tftp_clear_client() {client_ip = -1; client_port  = -1; client_block = -1;}


/*  send tftp error message 
    ip   = destination ip address
    post = destination UDP port
    code = TFTP error code 
*/

int tftp_send_err(unsigned long ip, unsigned short port, unsigned short code)
{
	struct tftphdr *hdr;
	struct sk_buff *skb;
	char * mes;
	int len, pos;

	switch(code) {
	    case EUNDEF		: mes = ""; break;
	    case ENOTFOUND	: mes = "file not found"; break;
	    case EACCESS	: mes = "access violation"; break;
	    case ENOSPACE	: mes = "disk full"; break;
	    case EBADOP		: mes = "illegal TFTP operation"; break;
	    case EBADID		: mes = "unknown transfer ID"; break;
	    case EEXISTS	: mes = "file already exists"; break;
	    case ENOUSER	: mes = "no such user"; break;
	    default		:
		return -1;
	}

	len = 0; 
	while(*(mes + len++)) ;

	skb = alloc_skb(ETH_FRAME_LEN);
	udp_skb_reserve(skb);
	hdr = (struct tftphdr *)skb_put(skb, sizeof(struct tftphdr) + len);
	
	hdr->th_opcode = htons(ERROR);
	hdr->th_code   = htons(code);
	pos = 0;
	while(pos < len) {
	    hdr->th_data[pos++] = *(mes + pos);
	}

	return udp_send(skb, ip, TFTP, port);
}

int tftp_send_ack(struct tftphdr *tftp_hdr, int block)
{
	struct tftphdr *tftp_ack;
	struct sk_buff *skb;

	skb = alloc_skb(ETH_FRAME_LEN);
	udp_skb_reserve(skb);
	tftp_ack = (struct tftphdr *)skb_put(skb, sizeof(struct tftphdr));
	tftp_ack->th_opcode = htons(ACK);
	tftp_ack->th_block = htons(block);

	return udp_send(skb, client_ip, TFTP, client_port);
}

int tftp_rcv_wrq(struct sk_buff *skb)
{
	struct tftphdr *tftp_hdr;

	client_ip = ip_get_source_ip(skb);
	client_port = udp_get_source_port(skb);

	tftp_hdr = (struct tftphdr *)skb->data;
	client_block = 1;
	if(tftp_send_ack(tftp_hdr, 0) == -1) {
		tftp_clear_client();
		return -1;
	}
	return tftp_put_begin();
}

int tftp_rcv_data(struct sk_buff *skb)
{
	struct tftphdr *tftp_hdr;
	unsigned long sip;
	unsigned short sport;
	int len;

	sip   = ip_get_source_ip(skb);
        sport = udp_get_source_port(skb);
	if (client_ip != sip || client_port != sport) {
		tftp_send_err(sip, sport, EBADID);
		return -1;
	}    

	tftp_hdr = (struct tftphdr *)skb->data;
	if (client_block == ntohs(tftp_hdr->th_block)) {
		len = skb->len - sizeof(struct tftphdr);
		if (tftp_put(tftp_hdr->th_data, len)) {
			tftp_send_err(sip, sport, ENOSPACE);
			return -1;
		}

		tftp_send_ack(tftp_hdr, client_block);
		client_block++;

		if (len < 512) {
			tftp_put_end();
			tftp_clear_client();
		}

	} else if (client_block > ntohs(tftp_hdr->th_block)) {

		return tftp_send_ack(tftp_hdr, ntohs(tftp_hdr->th_block));

	} else {

		return tftp_send_ack(tftp_hdr, client_block);
	}

	return 0;
}


int tftp_rcv_packet(struct sk_buff *skb)
{
	struct tftphdr *tftp_hdr;
	
	tftp_hdr = (struct tftphdr *)skb->data;

	switch (ntohs(tftp_hdr->th_opcode)) {

	case RRQ:
		break;
	case WRQ:
		tftp_rcv_wrq(skb);
		break;
	case DATA:
		tftp_rcv_data(skb);
		break;
	case ACK:
		break;
	case ERROR:
		break;
	default:
		break;
	}

	return 0;
}
