#include "tftpput.h"
#include "bioscall.h"
#include "console.h"
#include "utils.h"
#include "config.h"
#include "board.h"

#if 0
#define TFTP_BUF_BASE	(BUF_ADDR | NOCACHE_BIT)
#define TFTP_BUF_LIMIT	(TFTP_ADDR | NOCACHE_BIT)
#endif

#if 1
#define TFTP_BUF_BASE	(BUF_ADDR)
#define TFTP_BUF_LIMIT	(TFTP_ADDR)
#endif

extern unsigned long tftp_mode;
extern unsigned long tftp_param;
extern int tftp_break;
extern int tftp_datalen;

static unsigned char *buf;
static int data_len;
static int buf_size = 0; 
static int put_begin = 0;

int tftp_put_begin(void)
{
	put_begin = 1;
	
	switch (tftp_mode) {
	case BOOT_LOAD_IMAGE:
	case BOOT_UPDATE_IMAGE:
		printf("\r\nStarting the TFTP download...\r\n");
		buf = (unsigned char *)TFTP_BUF_BASE;
		buf_size = TFTP_BUF_LIMIT - TFTP_BUF_BASE;
		if(tftp_mode == BOOT_UPDATE_IMAGE) {
			struct partition_struct * part = 
			    (struct partition_struct *) tftp_param;
			if(buf_size > part->size)  
			    buf_size = part->size; 
		}
		data_len = 0;
		break;
	default:
		put_begin = 0;
		break;
	}

	return 0;
}

int tftp_put(unsigned char *data, int len)
{
	static int count = 0;

	if (!put_begin)
		return -1;

	if (count == 0)
		printf(".");
	count += len;
	if (count >= 32 * 1024)
		count = 0;

	switch (tftp_mode) {
	case BOOT_LOAD_IMAGE:
	case BOOT_UPDATE_IMAGE:
		if (data_len + len > buf_size) {
			printf("\r\nTFTP buffer overflow, stop!\r\n");
			tftp_break = 1;
			return -1;
		}
		memcpy(buf + data_len, data, len);
		data_len += len;
		return 0;
	default:
		return 0;
	}

	return 0;
}

static int update_image(void)
{
	struct partition_struct * partition =
	    (struct partition_struct *)tftp_param;
	unsigned long offset, l;
	

	printf("Update Image to Disk ...");

	offset = partition->offset;
	if(bios_flash_erase(offset, partition->size) == -1) {
	    printf("Flash erase Error\r\n");
	    return -1;
	}
	if(bios_flash_write(offset, data_len, &l, 
	    (unsigned char *)TFTP_BUF_BASE) == -1) {
	    printf("Flash write Error\r\n");
	    return -1;
	}     
	printf("Done\r\n");

	if (offset == bios_bios_offset()) {
		printf("\r\nBIOS Updated, rebooting...\r\n");
		bios_reboot();
	}

	return 0;
}

int tftp_put_end(void)
{
	if (!put_begin)
		return -1;
	
	printf("\r\n");
	put_begin = 0;
	tftp_break = 1;

	switch (tftp_mode) {
	case BOOT_LOAD_IMAGE:
		tftp_datalen = data_len;
		break;
	case BOOT_UPDATE_IMAGE:
		if(update_image() == -1)
		    return -1;
		break;
	default:
		break;
	}

	return 0;
}
