#include "skbuff.h"
#include "eth.h"
#include "ip.h"
#include "udp.h"
#include "tftp.h"
#include "utils.h"

extern unsigned long local_ip;	/* from ip level */

int udp_init(void)
{
	return 0;
}

/*  cacl udp check sum
    dip = destination IP address 
    sip = source IP address 
    udph = udp header pointer 
    udpl = udp length, includes header and data   
    based on corresponding function from tcpdump-3.8.1
*/

static unsigned short udp_csum(unsigned char * udph, unsigned int udpl,
    unsigned long dip, unsigned long sip )
{
 unsigned short * ptr = (unsigned short *) udph;
 unsigned long sum = 0;
 struct {
    unsigned long src;
    unsigned long dst;
    unsigned char mbz;
    unsigned char proto;
    unsigned short len;
 } ph = { htonl(sip), htonl(dip), 0, UDP, htons(udpl) };

 while(udpl > 1) {
    sum += *ptr++;
    udpl -= 2;
 }

 if(udpl == 1) {
    sum += htons(*((unsigned char *) ptr) << 8);
 }

 ptr = (unsigned short *) & ph;
 udpl = 6;
 while(udpl-- > 0) {
    sum += *ptr++;
 }

 while(sum >> 16)
    sum = (sum & 0xffff) + (sum >> 16);
 
 return ~sum;
}

int udp_rcv_packet(struct sk_buff *skb)
{
	struct udphdr *udp_hdr = (struct udphdr *)(skb->data);
	unsigned long sum, src;
	
	if(udp_hdr->check) {
	    src = ip_get_source_ip(skb);
	    sum = udp_csum((unsigned char *) udp_hdr, ntohs(udp_hdr->len), 
		    local_ip, src);
	    if(sum != 0) {
		return 0;
	    }
	}

	skb->len = ntohs(udp_hdr->len);
	skb_pull(skb, sizeof(struct udphdr));

	if (ntohs(udp_hdr->dest) == TFTP)
		tftp_rcv_packet(skb);

	return 0;
}


int udp_send(struct sk_buff *skb, unsigned long ip,
	unsigned short source, unsigned short dest)
{
	struct udphdr *udp_hdr;

	udp_hdr = (struct udphdr *)skb_push(skb, sizeof(struct udphdr));
	udp_hdr->source = htons(source);
	udp_hdr->dest = htons(dest);
	udp_hdr->len = htons(skb->len);
	udp_hdr->check = 0;

	udp_hdr->check = udp_csum((unsigned char *) udp_hdr, skb->len, ip, local_ip);

	return ip_send(skb, ip, UDP);
}

void udp_skb_reserve(struct sk_buff *skb)
{
	ip_skb_reserve(skb);
	skb_reserve(skb, sizeof(struct udphdr));
}

unsigned short udp_get_source_port(struct sk_buff *skb)
{
	struct udphdr *udp_hdr;
	struct iphdr  *ip_hdr; 
        ip_hdr  = (struct iphdr  *)(skb->buf + ETH_HLEN);
	udp_hdr = (struct udphdr *)((char *)ip_hdr + (ip_hdr->ihl*4));
	return ntohs(udp_hdr->source);
}
