#include <time.h>
#include "bios_ltu.h"

// symbol name for bin2c util ////////////////////////////////////////////////
char symbol_name[256] = "bin2c";  

unsigned int file_offset  = 0;
unsigned int file_size    = 0;

// bin2c utils usage /////////////////////////////////////////////////////////

int bin2c_usage(int to)
{
 if(to == TO_BIN2C) {
    fprintf (
      stdout	,
      "Usage : bin2c [options] <input file> [<output file>]\n"
      "where options may be:\n"
      "--symbol-name	- symbol name\n"
      "%s\n%s\n%s\n"		,	
	VERBOSE_USAGE		,	// verbose options  
	RECURSE_USAGE		,	// recurse options 
	SORT_FILE_USAGE			// sort file options 
    );
 }

//
 return(1);
}

// process option symbol ///////////////////////////////////////////////////// 

int opt_symbol(char * args[])
{
 if(args == 0 || args[0] == 0) {
    return(0);
 } 

 if( strcmp(args[0], "--symbol-name") == 0) {
    int dlen;
    if(*args[1] == '\0') {
	fprintf( stderr, 
	  "\nOption [--symbol-name] require string argument\n"
	);
	exit(0);
    }

    dlen = sizeof(symbol_name) - 1;
    strncpy(symbol_name, args[1], dlen);
    symbol_name[dlen] = '\0';
    return(1);
 }
 return(0);
}

// bin2c function ////////////////////////////////////////////////////////////  

int bin2c_func(int to, int argc, char * args[])
{
 if(verbose_mode >= VERBOSE_MODE_NORMAL) { 
// print programm name in not quiet mode  
    if(to == TO_BIN2C) {
// dump file fo unsigned char array //////////////////////////////////////////
	fprintf(stdout, "BIN2C Convert bin file to 'C' unsigned char []\n");  
    }
 }

// process input options /////////////////////////////////////////////////////
 int argp;				// argument number 
 for(argp = 1; argp < argc; ++argp) {  
    char * str = args[argp];
    if( opt_recurse(str) ||		// process recurce option 
	opt_sort_by(str)		// process sourt option 
    ) {
	continue;
    }

    if(*(str + 0) != '-') {		// not option 
	break;				// exit from loop
    }

    if(to == TO_BIN2C) {
	if(opt_symbol(args+argp) == 1) {
	    argp += 1;
	    continue;
	}
    }

//
    int * val = 0;


    if( strcmp(str, "--file-size") == 0) { 
    } 

    if(val == (int *) -1) {		// allready processed
    } else 	
    if(val != 0) {			// process int option 
	if(argp + 1 >= argc) {
	    fprintf(stderr, "\nOption [%s] require numeric argument", 
	        args[argp]
	    );
	    exit(0);
	} else {
	    argp += 1;
	    opt_int(val, args[argp]);
	}
    } else {
	opt_unknown(str);		// and exit 
	exit(0);
    }
 }

// 
 char ** inam;
 inam = opt_iname( args[argp] );
 if(inam == 0) {
    fprintf(stderr, "\nInternal error\n");
    exit(1);
 }

// 
 char ** onam;
 if( argp + 1 < argc ) {
    onam = opt_oname(args[argp + 1], inam);
 } else {
    onam = 0;
 }

// 
 int fn;
 fn = 0;
 while( *(inam + fn) != 0 ) {
// 
    if(verbose_mode >= VERBOSE_MODE_NORMAL) { 
    // print input & output file name 
	if(fn > 0) {
	    fprintf(stdout, "\n" );
	}
	    fprintf(stdout, "input  file   = [%s]\n", *(inam + fn) );
	if(onam != 0) {
	    fprintf(stdout, "output file   = [%s]\n", *(onam + fn) );
	}
    }


    FILE * fi = ifile_open(inam, fn, "r+b");	// input file open
    FILE * fo = ofile_open(onam, fn, "w+t");	// output file open 

    unsigned int filen = 0;		// input file length 
    unsigned int folen = 0;		// output file length
    int wrvl;				// writed bytes to output file 

    if(to == TO_BIN2C) {
// bin2c title ///////////////////////////////////////////////////////////////
	time_t tval = time(0); 		// read current time 
	char * tstr = ctime(&tval);	// convert time to string 

	wrvl = fprintf(fo,
	    "/* autogenerated file -- DO NOT EDIT! */ \n"
	    "/* autogenerated by: bin2c \n"
	    "	at %s \n"
	    "*/ \n"
	    "\n"
	    "const unsigned char %s[]= { \n"
	    , tstr, symbol_name
	);

	if(wrvl >= 0) {			// calc output file length
	    folen += wrvl;    
	}
    }

    unsigned char ch;
    ch = fgetc(fi);			// read byte 
    while(feof(fi) == 0) {		// while not file eof 
        filen++;			// read bytes count 
//
	if(to == TO_BIN2C) {
// bin2c util ////////////////////////////////////////////////////////////////  
	    if(filen != 1) {		// print array separator
		wrvl = fprintf(fo,", ");
		if(wrvl >= 0) {		// calculate output file  
		    folen += wrvl;    	// length
		}
	    }

	    if( ((filen - 1) % 8) == 0 ) {
	    // end of line 
    		if(filen != 1) {
		    wrvl = fprintf(fo, "\n");
		    if(wrvl >= 0) {	// calculate output file  
			folen += wrvl;  // length  
		    }

		    if( ((filen - 1) % 256) == 0 ) {
			wrvl = fprintf(fo, "/* end of block */ \n"); 
			if(wrvl >= 0) {
			    folen += wrvl;    
			}
		    }
		}
		wrvl = fprintf(fo, "   ");
		if(wrvl >= 0) {
		    folen += wrvl;    
		}
	    }
	    wrvl = fprintf(fo, "0x%02x", ch);
	    if(wrvl >= 0) {
		folen += wrvl;    
	    }
	}

//
	ch = fgetc(fi);			// read byte 
    }	

// 
    if(to == TO_BIN2C) {
	wrvl = fprintf(fo, 
	    "\n"
	    "}; \n\n"
	    "/* Note: %d characters output */ \n\n"
	    "/* end of file */ \n\n", 
	    (int) filen
	);	    
//
	if(wrvl >= 0) {
	    folen += wrvl;    
	}
    }

    if(verbose_mode >= VERBOSE_MODE_VERBOSE) { 
	fprintf(stdout, "input length  = [%d]\n", filen );
	if(onam != 0) {
	fprintf(stdout, "output length = [%d]\n", folen );
	}
    }
// 
    ifile_close(fi);			// close input file 
    ofile_close(fo);			// close output file 
    fn++; 
 }

 if(inam != 0) {			// free input names array 
    name_free(inam);
 }

 if(onam != 0) {			// free output names array
    name_free(onam);
 }

 return(1);
}


