#include <sys/stat.h>
#include <time.h>
#include <dirent.h>
#include <unistd.h>
#include "bios_ltu.h"

// process int option ///////////////////////////////////////////////////////

int opt_int(int * val, const char * str)
{
 if(val == 0 || str == 0) {
    return(0);
 }

 if( *str == 'x' || *str == 'X') {
    *val = strtoul(str + 1, 0, 16); 
 } else 
 if( *str == '0' && *(str + 1) == 'x' || *(str + 1) == 'X') {
    *val = strtoul(str + 2, 0, 16); 
 } else {
    *val = atoi(str);	// option value
 }
//
 return(1); 
}

// process recurse option ////////////////////////////////////////////////////

int opt_recurse(const char * str)
{
 if(str == 0) {				// null pointer
    return(0);
 }

 if(strcmp(str, "--recurse") == 0 || strcmp(str, "-r") == 0) {
    recurse_mode = 1; 			// set recurse mode 
    return(1);				// option str is recurse option
 } else {
    return(0);				// not recurse option
 }
}

// process verbose option ////////////////////////////////////////////////////

int opt_verbose(const char * str)
{
 if(str == 0) {
    return(0);
 }

 if(strcmp(str, "--verbose") == 0) {
    verbose_mode = VERBOSE_MODE_VERBOSE; 
 } else 
 if(strcmp(str, "--quiet") == 0) {
    verbose_mode = VERBOSE_MODE_QUIET; 
 } else {
    return(0);
 }
//
 return(1);
}


// process replace option ////////////////////////////////////////////////////

int opt_replace(const char * str)
{
 if(str == 0) {				// null pointer
    return(0);	
 }

 if( strcmp(str, "--replace" ) == 0) {
    replace_mode = 1;
    return(1);
 }
//
 return(0);
}

// process sort order option /////////////////////////////////////////////////

int opt_sort_by(const char * str)
{
 if(str == 0) {				// null pinter 
    return(0);
 }

 if(strcmp(str, "--no-sort"     ) == 0) {
    sort_mode = SORT_TYPE_UNSORTED; 	// no sort files
 } else 
 if(strcmp(str, "--sort-by-name") == 0) {
    sort_mode = SORT_TYPE_NAME;  	// sort files by name
 } else 
 if(strcmp(str, "--sort-by-ext" ) == 0) {
    sort_mode = SORT_TYPE_EXTENSION;	// sort files by file extension 
 } else 
 if(strcmp(str, "--sort-by-time") == 0) {
    sort_mode = SORT_TYPE_TIME;		// sort files by time 
 } else 
 if(strcmp(str, "--sort-by-size") == 0) {
    sort_mode = SORT_TYPE_SIZE;		// sort files by file size
 } else {
    return(0);				// not sort option 
 }

// 
 return(1);				// sort option
}

// print error message and exit //////////////////////////////////////////////

int opt_unknown(const char * str)
{
 if(str != 0) {
    fprintf(stderr, 
      "\nUnknown option [%s]\n", str
    ); 
 }
 exit(1);
}

// return null terminated array of input files names /////////////////////////

char ** opt_iname(const char *str)
{
 if(recurse_mode == 0) {
// not recurse mode => sizeof array is 2 ///////////////////////////////////// 
    char ** nam = (char **) malloc( sizeof(char *) * 2);

    if(nam == 0) {
	fprintf(stderr, "\nMemory alloc error.\n");
	exit(1);
    }
    char * tmp = (char *) malloc ( sizeof(char) * (strlen(str) + 1) );  
    if(tmp == 0) {
	fprintf(stderr, "\nMemory alloc error.\n");
	exit(1);
    }
    strcpy(tmp, str);
    *(nam + 0) = tmp;			// file name 
    *(nam + 1) = 0;			// array terminator
//
    return(nam);
 }   

// recurse mode //////////////////////////////////////////////////////////////
 DIR * dir = opendir(str);		// open directory 
 if(dir == 0) {				// open directory error 
    fprintf(stderr, 			// print error 
      "\nOpen directory error [%s].\n", str
    );
    exit(1);
 }

 struct dirent * de = readdir(dir);	// read first direntry 
 int cnt = 0;				// count of files  
 while(de != 0) {
    if(de->d_type == DT_REG) {
        cnt += 1;			// calc files count 
    }
    de = readdir(dir);
 }
 closedir(dir);

 char ** nam = (char **) malloc( sizeof(char *) * (cnt + 1) ); 
 if(nam == 0) {
    fprintf(stderr, "\nMemory alloc error\n");
    exit(1);
 }

 int fn;				// file number 
 for(fn = 0; fn < (cnt + 1); ++fn) {	// clear array 
    *(nam + fn) = 0; 
 }

 struct stat * sbf = 0;

 if(
    (sort_mode == SORT_TYPE_TIME) ||	// sort files by time 
    (sort_mode == SORT_TYPE_SIZE)	// sort files by file size
 ) {
// need stat system call /////////////////////////////////////////////////////
    sbf = (struct stat *) malloc( sizeof(struct stat) * cnt );

    if(sbf == 0) {
	fprintf(stderr, "\nMemory alloc error\n");
	free(nam);
	exit(1);
    }    

    memset(sbf, sizeof(struct stat) * cnt, 0);
 }

 dir = opendir(str);
 if(dir == 0) {
    fprintf(stderr, 			// print error 
      "\nOpen directory error [%s].\n", str
    );
    exit(1);
 }

// read directory and fill name & stat array ///////////////////////////////// 
 de = readdir(dir);			// read first dir entry
 fn = 0;				// start file number 
 while(de != 0) {
    if(de->d_type != DT_REG) {		// not regular file 
	de = readdir(dir);		// read next file 
	continue;
    }

// regular file //////////////////////////////////////////////////////////////
    if(fn >= cnt) {			// out of array 
	break;
    }

    int len = strlen(de->d_name);
    *(nam + fn) = (char *) malloc( strlen(str) + 1 + len + 1 );
// 
    if( *(nam + fn) == 0) {
	fprintf(stderr, "\nMemory alloc error\n");
	exit(1);
    }
    strcpy( *(nam + fn), str );		// directory name 
    strcat( *(nam + fn), "/");		// separator  
    strcat( *(nam + fn), de->d_name);	// file name 

    if(sbf != 0) {
// defined sbf (status buffer) - read status /////////////////////////////////
	if(stat( *(nam + fn), sbf + fn) == 0) {
	    fn++;			// increment file index
	} else {
	// read file status error => skip file ///////////////////////////////
	    free( *(nam + fn) );	// free file name 
	    *(nam + fn) = 0;		// clear name pointer 
	}
    } else {
	fn++;
    }
  de = readdir(dir);			// read next file name 
 }
 closedir(dir);				// close readed dir 
 cnt = fn;				// set files count 

 if(sort_mode != SORT_TYPE_UNSORTED) {  
// sort files ////////////////////////////////////////////////////////////////
    int n0;				
    for(n0 = 0; n0 + 1 < cnt; ++n0) {
	int n1;				
	for(n1 = n0 + 1; n1 < cnt; ++n1) { 

	    int cmp = 0; 

	    struct stat * sb0;		// file 0 status buffer	
	    if(sbf != 0) {
		sb0 = sbf + n0;
	    } else {
		sb0 = 0;
	    } 

	    struct stat * sb1;		// file 1 status buffer
	    if(sbf != 0) {
		sb1 = sbf + n1;
	    } else {
		sb1 = 0;
	    }

	    if(sort_mode == SORT_TYPE_UNSORTED){ 
// no sort files /////////////////////////////////////////////////////////////
		cmp = 0;
	    } else 
	    if(sort_mode == SORT_TYPE_NAME) {  	 
// sort files by file name ///////////////////////////////////////////////////
		cmp = strcmp( *(nam + n0), *(nam + n1));
	    } else 
	    if(sort_mode == SORT_TYPE_EXTENSION) {
// sort files by file extension //////////////////////////////////////////////
		char * s0, * e0;	// 0 - file name, extension	
		int pos;		// current name position
		s0 = *(nam + n0);	// 0 file name 
		pos = strlen(s0) - 1;	// last file name character
		e0 = s0 + pos + 1;	// set extension to null character

		while(pos >= 0) {
	    	    if(*(s0 + pos) == '.') {	// found extension separator 
			e0 = s0 + pos + 1;	// set file extension position 
			break;
		    }

		    if(*(s0 + pos) == '/') {	// found dir separator 
		    	break;			// null extension
		    }
		    pos -= 1;		// prev character
		}

		char * s1, * e1;	// 1 file, extension	
		s1 = *(nam + n1);	// read first file name 
		pos = strlen(s1) - 1;	// last file name character
		e1 = s1 + pos + 1;	// set extension to null character

		while(pos >= 0) {
	    	    if(*(s1 + pos) == '.') {	// extension separator
	    		e1 = s1 + pos + 1;	// extract extension
			break;
		    }

		    if(*(s1 + pos) == '/') {	// directory separator
			break;			// 	
		    }

		    pos -= 1;
		}

		cmp = strcmp(e0, e1);	// compare extension
//
		if(cmp == 0) {
	    	    int l0, l1;
	    	    l0 = (e0 - s0); 	// file name 0 length
	    	    l1 = (e1 - s1); 	// file name 1 length
	    	    if(l0 < l1) { 
	        	cmp = -1;	// < 
	    	    } else 
	    	    if(l0 > l1) {
	    		cmp = 1; 	// >
		    } else {
			cmp = strncmp(s0, s1, l0);
		    }
		}
	    } else 
	    if(sort_mode == SORT_TYPE_SIZE) {  
// sort files by time ////////////////////////////////////////////////////////
		if(sb0 != 0 && sb1 != 0) {
	    	    if(sb0->st_size < sb1->st_size) {
	    		cmp = -1;
		    } else 
		    if(sb0->st_size > sb1->st_size) {
			cmp =  1;
		    } else {
			cmp =  0;
		    } 
		} else {
		    cmp = 0;
		}
	    } else 
	    if(sort_mode == SORT_TYPE_TIME) {
// sort files by file size ///////////////////////////////////////////////////
		if(sb0 != 0 && sb1 != 0) {
		    if(sb0->st_ctime < sb1->st_ctime) {
	    		cmp = -1;
		    } else 
		    if(sb0->st_ctime > sb1->st_ctime) {
			cmp =  1;
		    } else {
			cmp =  0;
		    } 
		} else {
		    cmp = 0;
		}
	    }

	    if(cmp > 0) { 
		char * tmp;
		tmp = *(nam + n0);
	        *(nam + n0) = *(nam + n1);
		*(nam + n1) = tmp;

    		if(sb0 != 0 && sb1 != 0) {
	    	    struct stat stmp;
	    	    stmp = *sb0;
	    	    *sb0 = *sb1;
	    	    *sb1 = stmp;
		}
	    } 
	}
    }	 
 }

 if(sbf != 0) {				// free stat buffer
    free(sbf);
 }

//
 return(nam);				// return name array  
}

// dnam = output directory name 
// inam = array of input file names

char ** opt_oname(const char * dnam, char ** inam) 
{
 if(inam == 0) {
    return(0);		
 }

// 
 if(dnam != 0) {
    DIR * dir;				// directory struct 
    dir = opendir(dnam);		// open directory struct 	
    if(dir == 0) {
// open dir error 
	char ** nam;
	char * str;
	nam = (char **) malloc( sizeof(char *) * 2);
	str = (char *)  malloc( sizeof(char) * (strlen(dnam) + 1) ); 
	if(nam == 0 || str == 0) {
	    fprintf(stderr, "\nMemory alloc error.\n");
	    exit(1);
	}
	strcpy(str, dnam);
        *(nam + 0) = str;
	*(nam + 1) = 0; 
	return(nam);
    }
    closedir(dir);
 } else {
    if(replace_mode != 1) {
	fprintf(stderr, "\nInternal error.\n");
	exit(1);
    }
 }    

// calc files count in input array inam  /////////////////////////////////////
 int cnt = 0;
 while( *(inam + cnt) != 0 ) {
    cnt++;
 }

// alloc array of strings ////////////////////////////////////////////////////
 char ** nam;
 nam = (char **) malloc( sizeof(char *) * (cnt + 1) );
 if(nam == 0) {
    fprintf(stderr,
      "\nMemory alloc error\n"	 
    );
    exit(1);
 }    
 
 int nm;
 for(nm = 0; nm <= cnt; ++nm) {
    *(nam + nm) = 0;	
 } 

 for(nm = 0; nm < cnt; ++nm) {
    char * str, * p;
    str = *(inam + nm);			// input file name  

    if(replace_mode == 0) { 
	p = strrchr(str, '/');		// search dir separator 
	if(p == 0) {
	    p = str;
	} else {
	    p++;			// skip dir separator  
	}
    

    // alloc file name 
	str = (char *) malloc( sizeof(char) * ( strlen(dnam) + 1 + strlen(p) + 1) );

    //
	if(str == 0) {
	    fprintf(stderr, "\nMemory alloc error\n");
	    exit(1);
	}

    // form output file name  
	strcpy(str, dnam);			// directory name 
	strcat(str, "/");			// dir separator 
	strcat(str, p  );			// file name 
    } else {
	p = str;
	str = (char *) malloc( sizeof(char) * (strlen(p) + 2 + 1) ); 
	if(str == 0) {
	    fprintf(stderr, "\nMemory alloc error\n");
	    exit(1);
	}
	strcpy(str, p);			// 
	strcat(str, ".~");		//suffix in replace mode 
    }

    *(nam + nm) = str;			// write to array
 } 
//
 return(nam);				// return array 				
}

// free string array /////////////////////////////////////////////////////////

int name_free(char ** nam)
{
 if(nam == 0) {				// null pointer to array  
    return(0);
 }

// 
 int nm;
 for(nm = 0; *(nam + nm) != 0; ++nm) {
    if( *(nam + nm) != 0 ) {
	free( *(nam + nm) );
	*(nam + nm) = 0;
    } else {
	break;
    }
 }      

 free(nam);
 return(1);
}


// open input file //////////////////////////////////////////////////////////

FILE * ifile_open(char ** inam, int fn, const char * md)
{
 if(inam == 0) {
    fprintf(stderr, 
      "\nFile open error. Null array\n"
    );
    exit(1);
 }

 if(md == 0) {
    md = "r+b";
 }

 if(*(inam + fn) == 0) {
    fprintf(stderr, 
      "\nFile open error. Null file name with index [%d]\n", fn
    );
    exit(1);
 }

 FILE * fi;
 fi = fopen( *(inam + fn), md );
 if(fi == 0) {
    fprintf(stderr, 
      "\nFile [%s] open error.\n", *(inam + fn)
    );
    exit(1);
 }
//
 return(fi);
}

// open output file //////////////////////////////////////////////////////////

FILE * ofile_open(char ** onam, int fn, const char * md)
{
 if(onam == 0) {
    return(stdout); 
 } else {
    if( *(onam + fn) == 0 ) {
        return(stdout); 
    } else {
	if(md == 0) {
	    md = "w+b";			// default file open mode 
	}

	FILE * fo = fopen( *(onam + fn), md);
    	if(fo == 0) {
	    fprintf(stderr, 
	      "\nFile [%s] open error. Mode=[%s]\n", *(onam + fn), md );
	    exit(1);
	} 
	return(fo);
    }
 }
}

// close input file //////////////////////////////////////////////////////////

int ifile_close(FILE * file) 
{
 if(file == 0) {
    return(0);
 }
 fclose(file);
 return(1);
}

int ofile_close(FILE * file)
{
 if(file == 0) {
    return(0);
 }
 if(file == stdout) {
    return(1);
 }
 fclose(file);
 return(1);
}

