#ifndef __TUTIL_H_
#define __TUTIL_H_

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>

// 
typedef unsigned char  U8;
typedef unsigned short U16;
typedef unsigned long  U32;

#define TUTIL_USAGE "TUTIL 1.1 (c) 2003,2004 Dmitriy Cherkashin dch@ucrouter.ru" 

// convert type /////////////////////////////////////////////////////////////
#define TO_BIN2C  	  0x00004000	// - bin2c util
#define TO_FILE_COMBINE   0x00010000	// - combine two input files to one output file 

extern int sort_mode;			// sort type 

// 
#define SORT_TYPE_UNSORTED  0 	// no sort files
#define SORT_TYPE_NAME      1  	// sort files by name
#define SORT_TYPE_EXTENSION 2   // sort files by file extension 
#define SORT_TYPE_TIME      3   // sort files by time 
#define SORT_TYPE_SIZE      4	// sort files by file size

#define SORT_FILE_USAGE     				\
"--no-sort       - no sort files\n"			\
"--sort-by-name  - sort files by name\n"		\
"--sort-by-ext   - sort files by file extension\n"	\
"--sort-by-time  - sort files by time\n"		\
"--sort-by-size  - sort files by file size\n"	

extern int recurse_mode; 		// recurse mode 
#define RECURSE_USAGE     				\
"--recurse       - input file name and output file name is directory\n"\
" -r             - recurse, short form of --recurse option"

extern int cr_lf_mode; 			// CR-LF mode  
#define CR_LF_MODE_DEFAULT 0	// default 
#define CR_LF_MODE_UNIX    1	// 0x0A only 
#define CR_LF_MODE_WIN     2	// 0x0A, 0x0D

#define CR_LF_USAGE     				\
"--cr-lf-default - default line termination\n" 		\
"--cr-lf-unix    - new line in Unix style\n" 		\
"--cr-lf-win     - new line in WIN style\n"

// replace mode ////////////////////////////////////////////////////////////// 

extern int replace_mode; 		// replace mode 

#define REPLACE_USAGE					\
"--replace       - replace input file(s)\n"


#define USAGE_OPT_NAME_LENGHT  16

// 
extern int verbose_mode;		// verbose mode
#define VERBOSE_MODE_QUIET   0 		// quiet mode 
#define VERBOSE_MODE_NORMAL  1 		// normal mode 
#define VERBOSE_MODE_VERBOSE 2 		// verbose mode 

// verbose mode usage
#define VERBOSE_USAGE     				\
"--quiet         - quiet mode\n" 			\
"--verbose       - verbose mode" 		

// 
extern char dos_char_set[];		// dos character set 
extern char win_char_set[];		// windows character set 
extern char koi_char_set[];		// koi-r character set

// options to cut-file, combine-file utils ///////////////////////////////////  
extern unsigned int file_offset;
extern unsigned int file_size;

extern char symbol_name[256];  

extern int    opt_recurse(const char * str);	// process recurse option 
extern int    opt_cr_lf(const char * str); 	// process CR LF option 	
extern int    opt_replace(const char * str); 	// process replace option 
extern int    opt_sort_by(const char * str);	// process sort order option 
extern int    opt_verbose(const char * str);	// process verbose option
extern int    opt_int(int * val, const char * str);// process int option
extern int    opt_symbol(char * args[]);	// process option symbol 

extern int    opt_unknown(const char * str);	// print error message and exit 
extern char **opt_iname(const char *str);
extern char **opt_oname(const char *str, char ** inam);
extern int    name_free(char ** nam);

extern char * program_name(int to); 		// return programm name 
extern int    program_const(const char *str);	// return to const

// open/close input/output file //////////////////////////////////////////////
extern FILE * ifile_open(char ** inam, int fn, const char * md);
extern FILE * ofile_open(char ** onam, int fn, const char * md);
extern int    ifile_close(FILE * file);		// close input file  
extern int    ofile_close(FILE * file);		// close output file 

// usage functions 
extern int    bin2c_usage(int to);
extern int    combine_usage(int to);

extern int    bin2c_func(int to, int argc, char * args[]);
extern int    combine_func(int to, int argc, char * args[]);
#endif 							// __TUTIL_H_
