#include <sys/stat.h>
#include <time.h>
#include "bios_ltu.h"

// fill character ////////////////////////////////////////////////////////////
static int fill_char = 0; 

// combine utils usage ///////////////////////////////////////////////////////

int combine_usage(int to)
{
 if(to == TO_FILE_COMBINE) {
    fprintf( stdout	,
      "\nUsage : combine_file [options] <input file 1> <input file 2> <output file>\n"
      "where options may be:\n"
      "%s\n"	    
      "If option 'recurse' is specified then <output file> must be\n"
      "directory, and one of files <input file 1>, <input file 2>\n"
      "must be directory.\n"
      "--file-offset	- <input file 2> offset \n"
      "--fill-char  	- fill character specification\n"
      "%s\n"	,
	RECURSE_USAGE,			// recurse options
	VERBOSE_USAGE			// verbose options
     );
 }
//
 return(1);
}

// combine files ///////////////////////////////////////////////////////////// 
// nam0 = file 0 name 
// nam1 = file 1 name 
// nam2 = file 2 name  

static int combine_files(char * nam0, char * nam1, char * nam2)
{
// print files names and input parametes /////////////////////////////////////
 if(verbose_mode >= VERBOSE_MODE_NORMAL) { 
    fprintf(stdout, "1 input file  = [%s]\n", nam0 );	// 1 file 
    fprintf(stdout, "2 input file  = [%s]\n", nam1 );	// 2 file 
    fprintf(stdout, " output file  = [%s]\n", nam2 );	// 
    fprintf(stdout, " offset = 0x%08x\n", file_offset); // 
    fprintf(stdout, " fill char    = 0x%02x\n", fill_char);
 }

// open 1-st input file 
 FILE * fi;
 fi = fopen( nam0, "r+b" ); 
 if(fi == 0) {
    fprintf(stderr, 
      "\nFile [%s] open error.\n", nam0 );
	exit(1);
 }

// open output file  
 FILE * fo;
 fo = fopen( nam2, "w+b" ); 
 if(fo == 0) {
    fprintf(stderr, 
     "\nFile [%s] open error.\n", nam2 );
       exit(1);
 }

 unsigned int fi0len = 0;		// input file 0 length 
 unsigned int fi1len = 0;		// input file 1 length 
 unsigned int folen = 0;
 unsigned char ch;
 int rvl = 1;     

 ch = fgetc(fi);			// read byte 
 while(feof(fi) == 0) {		// while not file eof 
    if(fi0len < file_offset) {
        folen++;
        fi0len++;
        if(fprintf(fo, "%c", ch) != 1) {
	    rvl = 0;
	}
    } else {
        break;
    }
    ch = fgetc(fi);			// read next byte 
 }

// 	
 ch = fill_char;			
 while(folen < file_offset) {
    folen++;
    if(fprintf(fo, "%c", ch) != 1) {	// padding
	rvl = 0;
    }
 }

//
 fclose(fi);

// open second input file
 fi = fopen( nam1, "r+b" ); 
 if(fi == 0) {
    fprintf(stderr, 
      "\nFile [%s] open error.\n", nam1 );
	exit(1);
 }

 ch = fgetc(fi);			// read byte 
 fi1len = 0; 
 while(feof(fi) == 0) {		// while not file eof 
    fi1len++;			// read bytes count 
    folen++;
    if(fprintf(fo, "%c", ch) != 0) {
        rvl = 1;
    }

    ch = fgetc(fi);			// read byte 
 }	

 if(verbose_mode > VERBOSE_MODE_NORMAL) { 
    fprintf(stdout, "1 file length = [%d]\n", fi0len );
    fprintf(stdout, "2 file length = [%d]\n", fi1len );
    fprintf(stdout, "output length = [%d]\n", folen );
 }

 if(fi != 0) {
    fclose(fi);
    fi = 0;  
 }        

 if(fo != 0 && fo != stdout) {
    fclose(fo);
    fo = 0;
 }
//
 return(rvl);
}

int combine_error(char * nam0, char * nam1, char * nam2)
{
 fprintf(stderr,
  "\nCombine files error, input files [%s],[%s],"
  "output file [%s]\n"	,
    nam0		, 
    nam1 		, 
    nam2
 );

//
 return(1);
}

// combine function //////////////////////////////////////////////////////////  

int combine_func(int to, int argc, char * args[])
{
//
 int rvl = 1;

 if(verbose_mode >= VERBOSE_MODE_NORMAL) { 
// print programm name  
    if(to == TO_FILE_COMBINE) {
	fprintf(stdout, "File combine\n");
    }
 }

// process input options /////////////////////////////////////////////////////
 int argp;				// argument number 
 for(argp = 1; argp < argc; ++argp) {  
    char * str = args[argp];
    if( opt_recurse(str) ) {		// process recurce option 
	continue;
    }

    if(*(str + 0) != '-') {		// not option 
	break;				// exit from loop
    }

//
    int * val = 0;
    if( strcmp(str, "--file-offset") == 0) { 
	if(to == TO_FILE_COMBINE) {
	    val = (int *) & file_offset; 
	}
    } else 
    if( strcmp(str, "--fill-char") == 0) { 
	if(to == TO_FILE_COMBINE) {
	    val = (int *) & fill_char; 
	}
    } 

    if(val == (int *) -1) {		// allready processed
    } else 	
    if(val != 0) {			// process int option 
// process int option
	if(argp + 1 >= argc) {
	    fprintf(stderr, "\nOption [%s] require numeric argument", 
	        args[argp]
	    );
	    exit(0);
	} else {
	    argp += 1;
	    opt_int(val, args[argp]);
	}
    } else {
	opt_unknown(str);		// and exit 
	exit(0);
    }
 }

// 
 if(to == TO_FILE_COMBINE) {
// combine utility ///////////////////////////////////////////////////////////
    if(argp + 2 >= argc) {
	combine_usage(to);
	exit(1);
    }

    char * nam0 = *(args+argp+0);
    char * nam1 = *(args+argp+1);
    char * nam2 = *(args+argp+2);

    if(recurse_mode == 0) {
	if(combine_files( nam0, nam1, nam2) == 0) {
	    combine_error(nam0, nam1, nam2);
	    rvl = 0;
	}
    } else {
	int d[3] = {0, 0, 0};
	int fn;
	for(fn = 0; fn < 3; ++fn) {
	    struct stat sbuf;
	    if( stat(*(args+argp+fn), & sbuf) != 0 ) {
		fprintf(stderr, "Read file/directory [%s] status error\n",
		    *(args +argp+fn));
		exit(1);
	    }
	    d[fn] = (sbuf.st_mode & S_IFDIR);
	}


	if(d[0]) {
	// 0 - directory
    	    if(d[1]) {
		fprintf(stderr, 
		  "[%s] is derectory and [%s] is directory.\n"
		  "One must be regular file.\n",
		    nam0, nam1
		);
		exit(1);
	    } else {
	    // 0 - dir, 1 - file 
		if(d[2] == 0) {
		    fprintf(stderr, 
			"Output file must be directory name"
		    );
		    exit(1);
		} 
		
		char ** inam = opt_iname( nam0 );
		char ** onam = opt_oname( nam2, inam);
	
		int fn;
		for(fn = 0; *(inam + fn) != 0; ++fn) {
		    if(combine_files( *(inam + fn), nam1, *(onam+fn)) == 0) {
			combine_error(*(inam + fn), nam1, *(onam+fn));
			rvl = 0;
			break;
		    }
		}       

		if(inam != 0) {		// free input files array
		    free(inam);
		}

		if(onam != 0) {
		    free(onam);
		}
	    }
	} else {
	// 0 - file 
	    if(d[1]) {
	    // 0 - file, 1 - directory  
		if(d[2] == 0) {
		    fprintf(stderr, 
			"Output file [%s] must be directory name\n",
    			 nam2 
		    );
		    exit(1);
		} 

		char ** inam = opt_iname( nam1 );
		char ** onam = opt_oname( nam2, inam);

		int fn;
		for(fn = 0; *(inam + fn) != 0; ++fn) {
		    if(combine_files( nam0, *(inam + fn), *(onam+fn)) == 0) {
			combine_error(nam0, *(inam + fn), *(onam+fn));

			rvl = 0;
			break;
		    }
		}       
	    } else {
	    // 0 - file, 1 -file 
		if(combine_files(*(args+argp),*(args+argp+1),*(args+argp+2)) == 0) {
	    	    combine_error(*(args+argp),*(args+argp+1),*(args+argp+2));
		    rvl = 0;
		}
	    }
	} 
    /* end recurse mode on */
    }
/* end TO_FILE_COMBINE */
 }
//
 return(rvl);
}
