#include "bios_ltu.h"

// general options ///////////////////////////////////////////////////////////

int recurse_mode  = 0; 		// recurse mode 
int sort_mode     = 0;		// sort type 
int cr_lf_mode    = 0; 		// CR-LF mode  
int replace_mode  = 0;		// if output file not spec, when use stdout as output
int verbose_mode  = 
  VERBOSE_MODE_NORMAL; 

// return default programm name //////////////////////////////////////////////

char * program_name(int to) 
{
 switch(to) {
    case TO_BIN2C        : return("bin2c");		// bin2c util
    case TO_FILE_COMBINE : return("combine_file");	

    default		 :
        return("");
 }
}

// conver execution file name to numeric value ///////////////////////////////

int program_const(const char *str)
{
 if(str == 0) {
    return(0);
 }

 if( strcmp(str, "bin2c") == 0 ) {
    return(TO_BIN2C);			// bin2c util 
 } else 
 if(
    (strcmp(str, "combine-file") == 0) || 
    (strcmp(str, "combine_file") == 0)
 ) {
    return(TO_FILE_COMBINE);
 }

 return(0);
}

// printf TODO note and exit ///////////////////////////////////////////////// 

void todo(void) 
{
    fprintf(stdout	, 
	"TODO\n"
	"\n"		
    );
//
    exit(1);
}

// print Change Log note and exit ////////////////////////////////////////////

void change_log(void)
{
    fprintf(stdout	, 
	"Change Log\n"
	"Version 1.1:\n"
	"  - added '--todo', '--change-log' options\n"
	"\n"	
    );
//
    exit(1);
}

//////////////////////////////////////////////////////////////////////////////
// print help message ////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

int usage(int to)
{
 switch(to) {    
    case TO_BIN2C  	:	// bin2c utility
	bin2c_usage(to);
	break;

    case TO_FILE_COMBINE:	// files combine utility
	combine_usage(to);
	break;

    default		:	
// print general help ////////////////////////////////////////////////////////
	fprintf(
	    stdout	,
          "--bin2c        - convert bin file to unsigned char array (in bios-lt style)\n"
          "--combine-file - combine files (in bios-lt style)\n"
          "\n"
	);
	break;
  }    
  return(1);
}

// TUTIL main function ///////////////////////////////////////////////////////

int main(int argc, char * args[])
{
 int to = 0; 

 int argh;
 for(argh = 1; argh < argc; ++argh) {
    // check quiet mode 
    if(strcmp(args[argh], "--quiet") == 0) {
	verbose_mode = VERBOSE_MODE_QUIET;
	break;
    } else 
    if(strcmp(args[argh], "--verbose") == 0) {
	verbose_mode = VERBOSE_MODE_VERBOSE;
	break;
    }
 }

 if(argh < argc) {
    while(argh + 1 < argc) {
	args[argh] = args[argh + 1];
	argh++;
    }
    argc -= 1;
 }

 if(verbose_mode >= VERBOSE_MODE_NORMAL) {
    fprintf(stdout, "\n%s\n", TUTIL_USAGE);
 }

// search in input args help request ///////////////////////////////////////// 
 for(argh = 1; argh < argc; ++argh) {
    if(strcmp(args[argh], "--help") == 0) {
	break;
    }
 }

 if(argh >= argc) {
// 
    argh = 0;				// help opt not found 

// help argument not found => process --todo, --change-log command ///////////    
// and recurse mode flag 
    int argp;
    for(argp = 1; argp < argc; ++argp) {
	if(strcmp(args[argp], "--todo") == 0) { 
	    todo(); 
	}
//
	if(strcmp(args[argp], "--change-log") == 0) {
	    change_log();
	}
    }
 }

 if(argc >= 1) {
// extract convert type from string to numeric value /////////////////////////
    char * str = strrchr(args[0], '/');
    if(str == 0) {
	str = args[0];
    } else {
	str++; 
    }
    to = program_const(str);
 }

 if(to == 0) {
// search prorgamm name in options ///////////////////////////////////////////
    int argn;
    for(argn = 1; argn < argc; ++argn) {
	if(strncmp(args[argn], "--", 2) == 0) { 
	    to = program_const(args[argn] + 2);
	    if(to != 0) {
	// to option found 

		{ break; }
	    }
	}
    }

    if(to == 0) {
	usage(0);
	exit(1);
    }
 }

 if(argh != 0) {			// help request
    usage(to);
    exit(1);
 }

 switch(to) {
    case TO_BIN2C	:	// bin2c utility
	bin2c_func(to, argc, args);
	break;

    case TO_FILE_COMBINE:	// file combine utility
	combine_func(to, argc, args);
	break;

    default: 
	fprintf(stderr, "\nUnsupported conversion type\n");
	break;
 }
//
 return(1);
}  

